#!/bin/sh

if [[ -z "$1" ]]; then
    echo "Usage: $0 <version>" 2>&1
    exit 1
fi

VERSION=$1

printf "\nPrepare directory and list of GPG keys used to sign the release... "

rm -f keys.txt *keyring.gpg* *pubring.gpg bitcoin-gpg.inc
tar -xzf bitcoin-${VERSION}.tar.gz --strip-components=3 bitcoin-${VERSION}/contrib/builder-keys/keys.txt

printf "done.\n"

printf "Attempting to download all listed GPG keys for release ${VERSION}... "

source=20

printf "" > bitcoin-gpg.inc

while read fingerprint keyholder_name; do

    # Download each key in its separate GPG keyring, 2 attempts with different key servers:
    gpg2 -q --logger-fd=1 --no-default-keyring --keyring ./bitcoin-${fingerprint}-keyring.gpg --keyserver hkps://keyserver.ubuntu.com --recv-keys ${fingerprint} > /dev/null
    gpg2 -q --logger-fd=1 --no-default-keyring --keyring ./bitcoin-${fingerprint}-keyring.gpg --keyserver hkps://keys.openpgp.org --recv-keys ${fingerprint} > /dev/null

    # If the GPG keyring is not empty, export the key in pubring format and check if it has been used for signing the release:
    if [ -s ./bitcoin-${fingerprint}-keyring.gpg ]; then

        gpg2 -q --logger-fd=1 --no-default-keyring --keyring ./bitcoin-${fingerprint}-keyring.gpg --export --export-options export-minimal ${fingerprint} > bitcoin-${fingerprint}-pubring.gpg

        # Verify the specific key in the signature file:
        gpg2 --no-options --no-default-keyring --keyring ./bitcoin-${fingerprint}-pubring.gpg --status-fd=1 --verify SHA256SUMS.asc SHA256SUMS 2>/dev/null | grep --quiet "^\[GNUPG:\] GOODSIG "

        if [ $? == 0 ]; then
            echo "# ${fingerprint} - ${keyholder_name}" >> bitcoin-gpg.inc
            ((source=source+1))
            echo "Source${source}: bitcoin-${fingerprint}-pubring.gpg" >> bitcoin-gpg.inc
        else
            rm -f bitcoin-${fingerprint}-pubring.gpg
        fi

        rm -f bitcoin-${fingerprint}-keyring.gpg*
    fi

done < keys.txt

# Cleanup:
rm -f keys.txt

printf "done.\n"

printf "List of valid keys used to sign the release:\n"
cat bitcoin-gpg.inc
